#!/usr/bin/env python

import os,sys,inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
import toolkit
from toolkit import KerasROOTClassification

def init_model(geneparam):
    
    nb_layers = geneparam['nb_layers']
    nb_neurons = geneparam['nb_neurons']
    activation = geneparam['activation']
    #optimizer = geneparam['optimizer']
    lr = geneparam['lr']
    decay = geneparam['decay']
    momentum = geneparam['momentum']

    filename = "/project/etp4/nhartmann/trees/allTrees_m1.8_NoSys.root"
    
    c = KerasROOTClassification("",
                                signal_trees = [(filename, "GG_oneStep_1545_1265_985_NoSys")],
                                bkg_trees = [(filename, "ttbar_NoSys"),
                                     (filename, "wjets_Sherpa221_NoSys"),
                                     (filename, "zjets_Sherpa221_NoSys"),
                                     (filename, "diboson_Sherpa221_NoSys"),
                                     (filename, "ttv_NoSys"),
                                     (filename, "singletop_NoSys")
                                ],
                                branches = ["jet1Pt", "jet1Phi==-999?0:jet1Phi", "jet1Eta==-999?0:jet1Eta",
                                            "jet2Pt", "jet2Phi==-999?0:jet2Phi", "jet2Eta==-999?0:jet2Eta",
                                            "jet3Pt", "jet3Phi==-999?0:jet3Phi", "jet3Eta==-999?0:jet3Eta",
                                            "jet4Pt", "jet4Phi==-999?0:jet4Phi", "jet4Eta==-999?0:jet4Eta",
                                            "jet5Pt", "jet5Phi==-999?0:jet5Phi", "jet5Eta==-999?0:jet5Eta",
                                            "jet6Pt", "jet6Phi==-999?0:jet6Phi", "jet6Eta==-999?0:jet6Eta",
                                            "jet7Pt", "jet7Phi==-999?0:jet7Phi", "jet7Eta==-999?0:jet7Eta",
                                            "jet8Pt", "jet8Phi==-999?0:jet8Phi", "jet8Eta==-999?0:jet8Eta",
                                            "lep1Pt", "lep1Phi", "lep1Eta", "nJet30",
                                            "met", "met_Phi"],
                                dumping_enabled=False,
                                optimizer="SGD",
                                layers=nb_layers,
                                nodes=nb_neurons,
                                activation_function=activation,
                                optimizer_opts=dict(lr=lr, decay=decay,
                                    momentum=momentum),
                                earlystopping_opts=dict(monitor='val_loss',
                                    min_delta=0, patience=2, verbose=0, mode='auto'),
                                selection="lep1Pt<5000", # cut out a few very weird outliers
                                weight_expr = "eventWeight*genWeight",
                                identifiers = ["DatasetNumber", "EventNumber"],
                                step_bkg = 100)
    return c

def train_and_score(geneparam):
    model = init_model(geneparam)

    model.train(epochs=20)
    model.evaluate()
    score = model.score

    return score[1]  # 1 is accuracy. 0 is loss.
